﻿Type=StaticCode
Version=1.86
@EndOfDesignText@
'--- This code module contains everything you need to show a grid on your activity.
'--- You need to have a panel on your activity which will act as a container to embed the grid.

'--- You can have as much grids as you like on the same activity or multiple activities,
'--- as long as you provide seperate ParentPanels and variables of type 'TGrid'.
'--- Every variable of type 'TGrid' needs to have a list of 'TColumn' variables as shown in the Main module in the sample project

'--- This code is 'as is' and may be spread freely. Use it in your private and/or commercial apps.
'--- It would be nice if you would add a reference to Sigurd Decroos or the website proapp.be.


'!!! P.S.: this code module uses the Reflection library for the NinePatchImages !!!

'--- Globals
#Region Globals
Sub Process_Globals
	'--- General Constants for the Grid
	Dim PADDING As Int: PADDING = 4dip
	Dim SPACING As Int: SPACING = 2dip
	Dim GRIDSIZE As Int: GRIDSIZE = 1dip
	Dim FONTSIZE As Int: FONTSIZE = 16
	Dim HEADERHEIGHT As Int: HEADERHEIGHT = 24dip: Dim HEADERTEXTCOLOR As Int: HEADERTEXTCOLOR = Colors.White
	Dim FOOTERHEIGHT As Int: FOOTERHEIGHT = 24dip: Dim FOOTERTEXTCOLOR As Int: FOOTERTEXTCOLOR = Colors.White
	Dim RECORDHEIGHT As Int: RECORDHEIGHT = 24dip: Dim RECORDTEXTCOLOR As Int: RECORDTEXTCOLOR = Colors.White
	'--- Change the following height to have smaller radiobuttons, buttons, checkboxes, ... 
	'--- Don't make it too small or the view will not be drawn correctly
	Dim INPUTVIEWHEIGHT As Int: INPUTVIEWHEIGHT = 48dip 

	'--- Nine Patch Images
	Dim RECORD_HEADER As String: RECORD_HEADER = "recordheader"
	Dim RECORD_EVEN As String: RECORD_EVEN = "recordeven"
	Dim RECORD_ODD As String: RECORD_ODD = "recordodd"
	Dim RECORD_FOOTER As String: RECORD_FOOTER = "recordfooter"
	Dim RECORD_DOWN As String: RECORD_DOWN = "recorddown"
	Dim RECORD_DISABLED As String: RECORD_DISABLED = "recorddisabled"

	'--- Column Types
	Dim GLABEL As Int: GLABEL = 1
	Dim GBUTTON As Int: GBUTTON = 10
	Dim GCHECKBOX As Int: GCHECKBOX = 14
	Dim GRADIOBUTTON As Int: GRADIOBUTTON = 17
	Dim GIMAGE As Int: GIMAGE = 20

	'--- Summary Types
	Dim SCOUNT As Int: SCOUNT = 1
	Dim SNUMBER_SUM As Int: SNUMBER_SUM = 2
	Dim SNUMBER_MIN As Int: SNUMBER_MIN = 3
	Dim SNUMBER_MAX As Int: SNUMBER_MAX = 4
	Dim SNUMBER_AVG As Int: SNUMBER_AVG = 5
	
	'--- Selection Types
	Dim SELECTMODENONE As Int: SELECTMODENONE = 0
	Dim SELECTMODEROWS As Int: SELECTMODEROWS = 1
	Dim SELECTMODEFIELDS As Int: SELECTMODEFIELDS = 2

	'--- Definition of a Color Rule
	Type TColorRule(Value1 As Object, Value2 As Object, TextColor As Int, IsNumeric As Boolean)

	'--- Definition of a Column
	Type TColumn (DisplayField As String, TagField As String, MinHeightField As String, _
		HeaderText As String, FooterText As String, CustomTextColor As Int, CustomHeaderTextColor As Int, CustomFooterTextColor As Int, _
		EventName As String, Value As String, Left As Int, Top As Int, IsSecondRow As Boolean, Width As Int, MinHeight As Int, ColumnType As Int, AutoHeight As Boolean, _
		Alignment As Int, CustomFontSize As Int, TrueText As String, FalseText As String, NullText As String, Icon As Bitmap, _
		CheckField As String, CheckValue As Object, SummaryType As Int, SummaryValue As Object, _
		NumberIntegersMin As Int, NumberFractions As Int, NumberGrouping As Boolean, ColorRules As List, CustomTypeFace As Typeface)
	
	'--- Definition of a Field
	Type TField (Column As TColumn, FieldView As View, FieldBackground As View, Text As String, Value As Object, IsAlternate As Boolean)
	'--- Definition of a row
	Type TRow (Fields As List, VisibleIndex As Int, DataIndex As Int, IsAlternate As Boolean)
	
	'--- Definition of a grid
	Type TGrid (Columns As List, Data As List, Rows As List, SelectionMode As Int, SelectedRows As List, SelectedFields As List, Left As Int, Top As Int, Width As Int, Height As Int, SortField As String, _
		CustomPadding As Int, CustomSpacing As Int, CustomGridSize As Int, CustomHeaderHeight As Int, CustomFooterHeight As Int, CustomRecordHeight As Int, _
		ParentPanel As Panel, SharedCanvas As Canvas, hscGrid As HorizontalScrollView, vscGrid As ScrollView, HeaderPanel As Panel, FooterPanel As Panel, _
		UseCustomLayout As Boolean, UseHorizontalScroll As Boolean, _
		WhereValue As Object, WhereField As String, ShowHeaders As Boolean, ShowFooters As Boolean, HeaderEventName As String, RecordEventName As String, RecordField As String, _
		Page As Int, PageSize As Int)
End Sub
#End Region

'--- The main grid routines
#Region Grid

#Region Initialize
Sub InitializeColorRule(objColorRule As TColorRule, IsNumeric As Boolean, Value1 As Object, Value2 As Object, TextColor As Int)
	objColorRule.Initialize
	objColorRule.IsNumeric = IsNumeric
	objColorRule.Value1 = Value1
	objColorRule.Value2 = Value2
	objColorRule.TextColor = TextColor
End Sub
'--- A sub for initializing the most used properties of a column. 
'--- A developer can easily create a InitializeColumn2 with other OR more properties If required
Sub InitializeColumn(objColumn As TColumn, DisplayField As String, TagField As String, HeaderText As String, EventName As String, Width As Int, ColumnType As Int, Alignment As Int)
	objColumn.Initialize
	objColumn.ColorRules.Initialize
	objColumn.DisplayField = DisplayField
	objColumn.TagField = TagField
	If HeaderText = "" Then
		objColumn.HeaderText = DisplayField
	Else
		objColumn.HeaderText = HeaderText
	End If
	objColumn.EventName = EventName
	objColumn.Width = Width
	If ColumnType = 0 Then
		objColumn.ColumnType = GLABEL
	Else
		objColumn.ColumnType = ColumnType
	End If
	If Alignment = 0 Then
		objColumn.Alignment = Gravity.LEFT + Gravity.CENTER_VERTICAL
	Else
		objColumn.Alignment = Alignment
	End If
	objColumn.CustomHeaderTextColor = HEADERTEXTCOLOR
	objColumn.CustomFooterTextColor = FOOTERTEXTCOLOR
	objColumn.CustomTextColor = RECORDTEXTCOLOR
	objColumn.CustomFontSize = FONTSIZE
	objColumn.CustomTypeFace = Typeface.DEFAULT
End Sub

Sub InitializeRow(objRow As TRow)
	objRow.Initialize
	objRow.Fields.Initialize
End Sub

'--- A sub for initializing the basic properties of the grid.
Sub InitializeGrid(objGrid As TGrid)
	objGrid.Initialize
	objGrid.Columns.Initialize
	objGrid.Data.Initialize
	objGrid.Rows.Initialize
	objGrid.SelectedRows.Initialize
	objGrid.SelectedFields.Initialize
End Sub
#End Region

#Region Add Grid
'--- This sub is responsible for drawing the main grid. Headers and footers if required.
Sub AddGrid(objGrid As TGrid)
	ClearPanel(objGrid.ParentPanel, False, False) '--- Erases everything inside the ParentPanel
	Dim intPosX, intPosY As Int ': intPosX = 0: intPosY = 0
	
	'--- If Custom properties are not set, set them to default
	If objGrid.CustomPadding = 0 Then objGrid.CustomPadding = PADDING
	If objGrid.CustomSpacing = 0 Then objGrid.CustomSpacing = SPACING
	If objGrid.CustomGridSize = 0 Then objGrid.CustomGridSize = GRIDSIZE
	If objGrid.CustomHeaderHeight = 0 Then objGrid.CustomHeaderHeight = HEADERHEIGHT
	If objGrid.CustomFooterHeight = 0 Then objGrid.CustomFooterHeight = FOOTERHEIGHT
	If objGrid.CustomRecordHeight = 0 Then objGrid.CustomRecordHeight = RECORDHEIGHT
	
	If objGrid.Width = 0 Then objGrid.width = objGrid.ParentPanel.Width
	If objGrid.Height = 0 Then objGrid.Height = objGrid.ParentPanel.Height
	
	'--- Horizontal scrolling required? If so, add a horizontal scrollview
	If objGrid.UseHorizontalScroll = True Then
		If objGrid.hscGrid.IsInitialized = False Then
			Dim hscGrid As HorizontalScrollView: hscGrid.Initialize(0, "")
			objGrid.hscGrid = hscGrid
		End If
	End If
	'--- Vertical scrollview initialized? If not, initialize it
	If objGrid.vscGrid.IsInitialized = False Then
		Dim vscGrid As ScrollView: vscGrid.Initialize(0)
		objGrid.vscGrid = vscGrid
	End If
	'--- Make a canvas for the ParentPanel, this canvas is used to calculate the width of the text of a field
	Dim objCanvas As Canvas: objCanvas.Initialize(objGrid.ParentPanel)
	objGrid.SharedCanvas = objCanvas
	
	'--- Header and Footer 'container' panel
	Dim pnlHeader As Panel: pnlHeader.Initialize(""): objGrid.HeaderPanel = pnlHeader
	If objGrid.FooterPanel.IsInitialized = False Then
		Dim pnlFooter As Panel: pnlFooter.Initialize(""): objGrid.FooterPanel = pnlFooter
	Else
		ClearPanel(objGrid.FooterPanel, True, False)
	End If

	'--- Add headers if required
	If objGrid.ShowHeaders = True Then
		'--- Placing of the scrollviews and header and footer panels
		If objGrid.UseHorizontalScroll = True Then
			objGrid.ParentPanel.AddView(objGrid.hscGrid, objGrid.Left, objGrid.Top, objGrid.Width, objGrid.Height)
			objGrid.hscGrid.Panel.AddView(pnlHeader, 0, 0, objGrid.hscGrid.Width, objGrid.CustomHeaderHeight)
			If objGrid.ShowFooters = True Then
				objGrid.hscGrid.Panel.AddView(objGrid.FooterPanel, 0, objGrid.Height - objGrid.CustomFooterHeight, objGrid.hscGrid.Width, objGrid.CustomFooterHeight)
				objGrid.hscGrid.Panel.AddView(objGrid.vscGrid, 0, objGrid.CustomHeaderHeight, objGrid.Width, objGrid.Height - objGrid.CustomHeaderHeight - objGrid.CustomFooterHeight)
			Else
				objGrid.hscGrid.Panel.AddView(objGrid.vscGrid, 0, objGrid.CustomHeaderHeight, objGrid.Width, objGrid.Height - objGrid.CustomHeaderHeight)
			End If
		Else
			objGrid.ParentPanel.AddView(pnlHeader, objGrid.Left, objGrid.Top, objGrid.Width, objGrid.CustomHeaderHeight)
			If objGrid.ShowFooters = True Then
				objGrid.ParentPanel.AddView(objGrid.FooterPanel, 0, objGrid.Height - objGrid.CustomFooterHeight, objGrid.Width, objGrid.CustomFooterHeight)
				objGrid.ParentPanel.AddView(objGrid.vscGrid, objGrid.Left, objGrid.Top + objGrid.CustomHeaderHeight, objGrid.Width, objGrid.Height - objGrid.CustomHeaderHeight - objGrid.CustomFooterHeight)
			Else
				objGrid.ParentPanel.AddView(objGrid.vscGrid, objGrid.Left, objGrid.Top + objGrid.CustomHeaderHeight, objGrid.Width, objGrid.Height - objGrid.CustomHeaderHeight)
			End If
		End If
		'--- Place header labels and header background buttons
		For i = 0 To objGrid.Columns.Size - 1
			Dim objColumn As TColumn: objColumn = objGrid.Columns.Get(i)
			If objColumn.IsSecondRow = False Then
				Dim lblHeader As Label: lblHeader.Initialize(""): lblHeader.Text = objColumn.HeaderText: lblHeader.Gravity = objColumn.Alignment: lblHeader.TextSize = objColumn.CustomFontSize: lblHeader.TextColor = objColumn.CustomHeaderTextColor
				pnlHeader.AddView(lblHeader, intPosX + objGrid.CustomSpacing, 0, objColumn.Width - objGrid.CustomSpacing * 2 - objGrid.CustomGridSize, objGrid.CustomHeaderHeight)

				'--- Background Header Panel
				Dim pnlSepHeader As Button: pnlSepHeader.Initialize(objGrid.HeaderEventName): pnlSepHeader.Tag = objColumn.DisplayField
				pnlHeader.AddView(pnlSepHeader, intPosX, 0, objColumn.Width - objGrid.CustomGridSize, objGrid.CustomHeaderHeight - objGrid.CustomGridSize)
				SetNinePatchButton(pnlSepHeader, RECORD_HEADER, RECORD_DOWN): pnlSepHeader.SendToBack

				'--- Calculate Next Header Position
				intPosX = intPosX + objColumn.Width
				If i < objGrid.Columns.Size - 1 Then intPosX = intPosX
			End If
		Next
		'--- Draw the footer
		RefreshFooter(objGrid)
		If objGrid.hscGrid.IsInitialized = True Then
			objGrid.hscGrid.Panel.Width = intPosX
		End If
		pnlHeader.Width = intPosX
		If objGrid.ShowFooters = True Then objGrid.FooterPanel.Width = intPosX
		objGrid.vscGrid.Width = intPosX
	Else
		objGrid.ParentPanel.AddView(objGrid.vscGrid, objGrid.Left, objGrid.Top, objGrid.Width, objGrid.Height)
	End If
	DoEvents
	objGrid.vscGrid.ScrollPosition = 0
	
	'--- Show the data, if present
	RefreshGridData(objGrid)
End Sub
#End Region

#Region Refresh Grid
'--- Refreshes the data of the grid, call this routine after filling the data or changing values
Sub RefreshGridData(objGrid As TGrid)
	Dim intPosX, intPosX2, intPosY As Int: intPosY = 0
	Dim blnAlternate As Boolean
	ClearScrollView(objGrid.vscGrid)
	
	objGrid.Rows.Initialize '--- Reset Row Information

	'--- Only refresh the footer values if no data present
	If objGrid.Data.IsInitialized = False OR objGrid.Data.Size = 0 Then
		RefreshFooter(objGrid)
		Return
	End If

	'--- Calculate start and end row indexes
	Dim intStartRow, intEndRow As Int
	If objGrid.PageSize <= 0 OR objGrid.Page < 0 Then
		intStartRow = 0: intEndRow = objGrid.Data.Size - 1
	Else
		intStartRow = objGrid.Page * objGrid.PageSize
		intEndRow = intStartRow + objGrid.PageSize - 1
		If intStartRow >= objGrid.Data.Size Then intStartRow = objGrid.Data.Size - objGrid.PageSize
		If intEndRow >= objGrid.Data.Size Then intEndRow = objGrid.Data.Size - 1
	End If
	
	'--- 'Fills' the grid
	Dim intVisibleIndex As Int: intVisibleIndex = 0
	For intRow = intStartRow To intEndRow
		intPosX = 0: intPosX2 = 0
		Dim objRecord As Map: objRecord = objGrid.Data.Get(intRow) '--- This is a map object representing a record of the data
		Dim fltRecordHeight, fltRecordHeight2 As Float: fltRecordHeight = 0: fltRecordHeight2 = 0
		
		Dim objRow As TRow: InitializeRow(objRow): objRow.IsAlternate = blnAlternate
		objRow.DataIndex = intRow: objRow.VisibleIndex = intVisibleIndex: intVisibleIndex = intVisibleIndex + 1
		objGrid.Rows.Add(objRow)

		For intColumn = 0 To objGrid.Columns.Size - 1 '--- Go through the defined columns
			Dim objColumn As TColumn: objColumn = objGrid.Columns.Get(intColumn): If objColumn.CustomFontSize = 0 Then objColumn.CustomFontSize = FONTSIZE
			Dim fltTextWidth, fltValueHeight As Float
			If objColumn.MinHeight = 0 Then '--- Minimum field height defined?
				fltValueHeight = objGrid.CustomRecordHeight
			Else
				fltValueHeight = objColumn.MinHeight ' objGrid.CustomRecordHeight + PADDING * 2
			End If
			Dim objValue As Object: objValue = objRecord.Get(objColumn.DisplayField) '--- Get the value of the record for this column
			Dim intLineCount As Int: Dim fltLineCount As Float
			Dim blnSecondRow As Boolean: blnSecondRow = objColumn.IsSecondRow '--- Is the field to be displayed on the second row?

			Dim objView As View '--- objView is of type View, this means all custom column types have to be derived from the view type. The view type has a tag property
			Dim intBaseX, intBaseY As Int
			If blnSecondRow = False Then '--- Calculate the horizontal position of the field, taking into account if it is second row or not
				intBaseX = intPosX + objGrid.CustomSpacing: intBaseY = intPosY
			Else
				intBaseX = intPosX2 + objGrid.CustomSpacing: intBaseY = intPosY + fltRecordHeight
			End If
			
			Dim objField As TField: objField.Initialize
			objField.Column = objColumn: objField.Value = objValue: objField.IsAlternate = blnAlternate
			objRow.Fields.Add(objField)

			'--- Initialize the column type as defined
			Select Case objColumn.ColumnType
				Case GLABEL
					Dim lblView As Label: lblView.Initialize(objColumn.EventName): objView = lblView: lblView.Gravity = objColumn.Alignment: lblView.TextSize = objColumn.CustomFontSize
					If objColumn.CustomTypeFace <> lblView.Typeface Then lblView.Typeface = objColumn.CustomTypeFace
					RefreshGLabel(lblView, objColumn, objValue)
					objField.Text = lblView.Text
				Case GIMAGE
					Dim imgView As ImageView: imgView.Initialize(objColumn.EventName): objView = imgView: imgView.Gravity = objColumn.Alignment
					If objValue <> Null Then
						imgView.Bitmap = objValue
					End If
				Case GRADIOBUTTON
					Dim rdbView As RadioButton: rdbView.Initialize(objColumn.EventName): objView = rdbView: rdbView.Gravity = objColumn.Alignment: rdbView.TextSize = objColumn.CustomFontSize
					If objColumn.CustomTypeFace <> rdbView.Typeface Then rdbView.Typeface = objColumn.CustomTypeFace
					rdbView.TextColor = GetColumnTextColor(objColumn, objValue): rdbView.TextSize = objColumn.CustomFontSize: If objValue <> Null Then rdbView.Text = objValue
					If objColumn.MinHeight = 0 Then objColumn.MinHeight = INPUTVIEWHEIGHT '--- Needed if the developer forgets to set the MinHeight for this ColumnType
					If objColumn.CheckValue <> Null AND objColumn.CheckValue = objRecord.Get(objColumn.CheckField) Then rdbView.Checked = True '--- AutoCheck if required and equal
					objField.Text = rdbView.Text
				Case GCHECKBOX
					Dim chkView As CheckBox: chkView.Initialize(objColumn.EventName): objView = chkView: chkView.Gravity = objColumn.Alignment: chkView.TextSize = objColumn.CustomFontSize
					If objColumn.CustomTypeFace <> chkView.Typeface Then chkView.Typeface = objColumn.CustomTypeFace
					chkView.TextColor = GetColumnTextColor(objColumn, objValue): chkView.TextSize = objColumn.CustomFontSize: If objValue <> Null Then chkView.Text = objValue
					If objColumn.MinHeight = 0 Then objColumn.MinHeight = INPUTVIEWHEIGHT '--- Needed if the developer forgets to set the MinHeight for this ColumnType
					If objColumn.CheckValue <> Null AND objColumn.CheckValue = objRecord.Get(objColumn.CheckField) Then chkView.Checked = True '--- AutoCheck if required and equal
					objField.Text = chkView.Text
				Case GBUTTON
					Dim btnView As Button: btnView.Initialize(objColumn.EventName): objView = btnView: btnView.Gravity = objColumn.Alignment: btnView.TextSize = objColumn.CustomFontSize
					If objColumn.CustomTypeFace <> btnView.Typeface Then btnView.Typeface = objColumn.CustomTypeFace
					btnView.TextColor = GetColumnTextColor(objColumn, objValue): btnView.TextSize = objColumn.CustomFontSize: If objValue <> Null Then btnView.Text = objValue
					If objColumn.Icon.IsInitialized = True Then 
						btnView.SetBackgroundImage(objColumn.Icon)
					Else '--- Set ninepatchimages to make the button look better with our GUI
					End If
					If objColumn.MinHeight = 0 Then objColumn.MinHeight = INPUTVIEWHEIGHT '--- Needed if the developer forgets to set the MinHeight for this ColumnType
					objField.Text = btnView.Text
			End Select
			
			'--- If column type is supported, the objView is initialized and can be placed in the grid
			If objView.IsInitialized = True Then
				objGrid.vscGrid.Panel.AddView(objView, intBaseX, intBaseY, objColumn.Width - objGrid.CustomSpacing * 2, fltValueHeight)
				'--- Calculate the height of the value or use the default
				If objColumn.AutoHeight = True Then '--- Support for autoheight of value
					fltValueHeight = GetMultiLineHeight(objGrid.SharedCanvas, objView, Typeface.DEFAULT) + PADDING
					If fltValueHeight < objColumn.MinHeight Then fltValueHeight = objColumn.MinHeight
				Else If objColumn.MinHeight = 0 AND objColumn.MinHeightField.Length = 0 Then
					fltValueHeight = objGrid.CustomRecordHeight
				Else If objColumn.MinHeightField.Length > 0 Then
					fltValueHeight = objRecord.Get(objColumn.MinHeightField)
				Else
					fltValueHeight = objColumn.MinHeight
				End If
				'--- Fill the tag property of the view, if a field has been provided in the column
				If objColumn.TagField.Length > 0 Then objView.Tag = objRecord.Get(objColumn.TagField)
				objView.Height = fltValueHeight
				objField.FieldView = objView
			End If

			'--- Keep track of the largest height of the row, so all backpanels are the same size, this is needed for autoheight
			If blnSecondRow = False Then
				If fltValueHeight > fltRecordHeight Then fltRecordHeight = fltValueHeight
			Else If objValue <> Null AND objValue <> "" Then '--- SecondRow has value
				If fltValueHeight > fltRecordHeight2 Then fltRecordHeight2 = fltValueHeight
			End If

			'--- Calculate horizontal position of next value per row
			If blnSecondRow = False Then
				intPosX = intPosX + objColumn.Width
				If intColumn < objGrid.Columns.Size - 1 Then intPosX = intPosX '+ objGrid.CustomGridSize
			Else
				intPosX2 = intPosX2 + objColumn.Width
				If intColumn < objGrid.Columns.Size - 1 Then intPosX2 = intPosX2 '+ objGrid.CustomGridSize
			End If
		Next

		
		'--- Background panels
		intPosX = 0: intPosX2 = 0
		For intColumn = 0 To objGrid.Columns.Size - 1
			Dim objColumn As TColumn: objColumn = objGrid.Columns.Get(intColumn)
			Dim objField As TField: objField = objRow.Fields.Get(intColumn)
			Dim blnSecondRow As Boolean: blnSecondRow = objColumn.IsSecondRow
			Dim pnlSeperator As Button: pnlSeperator.Initialize(objGrid.RecordEventName)
			If objGrid.RecordField.Length > 0 Then pnlSeperator.Tag = objRecord.Get(objGrid.RecordField)

			'--- Use Nine Patch Images, comment the following 3 lines to use the colors
			If blnAlternate = False Then: SetNinePatchButton2(pnlSeperator, RECORD_ODD, RECORD_DOWN, RECORD_DISABLED, RECORD_DOWN, RECORD_DISABLED)
			Else: SetNinePatchButton2(pnlSeperator, RECORD_EVEN, RECORD_DOWN, RECORD_DISABLED, RECORD_DOWN, RECORD_DISABLED) '  SetNinePatchDrawable(pnlSeperator, RECORD_EVEN)
			End If
			'--- Uncomment this if you want to use colors
'			If blnAlternate = False Then: pnlSeperator.Color = Colors.Gray
'			Else: pnlSeperator.Color = Colors.DarkGray
'			End If

			If blnSecondRow = False Then
				objGrid.vscGrid.Panel.AddView(pnlSeperator, intPosX, intPosY, objColumn.Width - objGrid.CustomGridSize, fltRecordHeight - objGrid.CustomGridSize)
			Else If objValue <> Null AND objValue <> "" Then '--- SecondRow has value
				objGrid.vscGrid.Panel.AddView(pnlSeperator, intPosX2, intPosY + fltRecordHeight, objColumn.Width - objGrid.CustomGridSize, fltRecordHeight2 - objGrid.CustomGridSize)
			End If
			pnlSeperator.SendToBack

			If blnSecondRow = False Then
				intPosX = intPosX + objColumn.Width
				If intColumn < objGrid.Columns.Size - 1 Then intPosX = intPosX '+ 1dip
			Else
				intPosX2 = intPosX2 + objColumn.Width
				If intColumn < objGrid.Columns.Size - 1 Then intPosX2 = intPosX2 '+ 1dip
			End If
			objField.FieldBackground = pnlSeperator
		Next
		
		blnAlternate = Not(blnAlternate) '--- Alternate the background image. Using an integer with MOD 10 for example allows you to use 10 different colored records to make a rainbow like grid pattern :)
		intPosY = intPosY + fltRecordHeight + fltRecordHeight2 '+ objGrid.CustomGridSize
	Next
	
	RefreshSelection(objGrid)
	
	RefreshFooter(objGrid) '--- Refresh the footer summary values

	objGrid.vscGrid.Panel.Height = intPosY '--- Set the height of the scrollview so everything becomes visible
End Sub

Sub GetColumnTextColor(objColumn As TColumn, objValue As Object) As Int
	If objColumn.ColorRules.Size = 0 Then
		Return objColumn.CustomTextColor
	Else
		Try
			For intRule = 0 To objColumn.ColorRules.Size - 1
				Dim objRule As TColorRule: objRule = objColumn.ColorRules.Get(intRule)
				If objRule.IsNumeric Then
					If objValue >= objRule.Value1 AND objValue <= objRule.Value2 Then
						Return objRule.TextColor
					End If
				Else
					If objValue = objRule.Value1 OR objValue = objRule.Value2 Then
						Return objRule.TextColor
					End If
				End If
			Next
			Return objColumn.CustomTextColor
		Catch
			Return objColumn.CustomTextColor
		End Try
	End If
End Sub

'--- Internal Sub for displaying and coloring a label, should not be used except in the grid module itself
Sub RefreshGLabel(lblView As Label, objColumn As TColumn, objValue As Object)
	lblView.TextColor = GetColumnTextColor(objColumn, objValue) ': lblView.TextSize = objColumn.CustomFontSize
	If objValue <> Null Then 
		If (objValue Is Float OR objValue Is Double) AND objColumn.NumberIntegersMin > 0 Then
			lblView.Text = NumberFormat2(objValue, objColumn.NumberIntegersMin, objColumn.NumberFractions, objColumn.NumberFractions, objColumn.NumberGrouping)
		Else
			lblView.Text = objValue
		End If
	Else
		lblView.Text = ""
	End If
End Sub

'--- This sub paints and refreshes the footer values
Sub RefreshFooter(objGrid As TGrid)
	If objGrid.ShowFooters = False Then Return

	'--- Reset the summary values to their standard
	Dim objSummary(objGrid.Columns.Size) As Object
	For intColumn = 0 To objGrid.Columns.Size - 1
		Dim objColumn As TColumn: objColumn = objGrid.Columns.Get(intColumn)
		Select Case objColumn.SummaryType
			Case SCOUNT, SNUMBER_SUM, SNUMBER_MIN, SNUMBER_MAX, SNUMBER_AVG: objSummary(intColumn) = 0
			Case Else: objSummary(intColumn) = ""
		End Select
		objColumn.SummaryValue = objSummary(intColumn)
	Next

	'--- Calculate start and end row indexes
	Dim intStartRow, intEndRow As Int
	If objGrid.PageSize <= 0 OR objGrid.Page < 0 Then
		intStartRow = 0: intEndRow = objGrid.Data.Size - 1
	Else
		intStartRow = objGrid.Page * objGrid.PageSize
		intEndRow = intStartRow + objGrid.PageSize - 1
		If intStartRow >= objGrid.Data.Size Then intStartRow = objGrid.Data.Size - objGrid.PageSize
		If intEndRow >= objGrid.Data.Size Then intEndRow = objGrid.Data.Size - 1
	End If
	
	If intStartRow >= 0 AND intEndRow > 0 Then
		'--- Summary Calculation
		For intRow = intStartRow To intEndRow
			Dim objRecord As Map: objRecord = objGrid.Data.Get(intRow) '--- This is a map object representing a record of the data
			For intColumn = 0 To objGrid.Columns.Size - 1 '--- Go through the defined columns
				Dim objColumn As TColumn: objColumn = objGrid.Columns.Get(intColumn)
				Dim objValue As Object: objValue = objRecord.Get(objColumn.DisplayField) '--- Get the value of the record for this column
				Select Case objColumn.SummaryType
					Case SCOUNT: objSummary(intColumn) = objSummary(intColumn) + 1
					Case SNUMBER_SUM: If objValue <> Null Then objSummary(intColumn) = objSummary(intColumn) + objValue
					Case SNUMBER_MIN: If intRow = 0 OR (objValue <> Null AND objSummary(intColumn) > objValue) Then objSummary(intColumn) = objValue
					Case SNUMBER_MAX: If intRow = 0 OR (objValue <> Null AND objSummary(intColumn) < objValue) Then objSummary(intColumn) = objValue
					Case SNUMBER_AVG: If objValue <> Null Then objSummary(intColumn) = objSummary(intColumn) + objValue
				End Select
			Next
		Next
	End If

	ClearPanel(objGrid.FooterPanel, False, False)
	'--- Summary, if pagesize is set, summary is calculated per page!
	Dim intPosX As Int
	For intColumn = 0 To objGrid.Columns.Size - 1
		Dim objColumn As TColumn: objColumn = objGrid.Columns.Get(intColumn)
		Select Case objColumn.SummaryType
			Case SNUMBER_AVG
				If objGrid.PageSize > 0 Then
					objSummary(intColumn) = objSummary(intColumn) / objGrid.PageSize
				Else
					objSummary(intColumn) = objSummary(intColumn) / objGrid.Data.Size
				End If
		End Select
		objColumn.SummaryValue = objSummary(intColumn)
		If objColumn.IsSecondRow = False Then
			Dim lblFooter As Label: lblFooter.Initialize(""): lblFooter.Text = objColumn.FooterText: lblFooter.Gravity = objColumn.Alignment: lblFooter.TextSize = objColumn.CustomFontSize: lblFooter.TextColor = objColumn.CustomFooterTextColor
			If objColumn.SummaryValue <> Null Then 
				If (objColumn.SummaryValue Is Float OR objColumn.SummaryValue Is Double) AND objColumn.NumberIntegersMin > 0 Then
					lblFooter.Text = lblFooter.Text & NumberFormat2(objColumn.SummaryValue, objColumn.NumberIntegersMin, objColumn.NumberFractions, objColumn.NumberFractions, objColumn.NumberGrouping)
				Else
					lblFooter.Text = lblFooter.Text & objColumn.SummaryValue
				End If
			End If
			objGrid.FooterPanel.AddView(lblFooter, intPosX + objGrid.CustomSpacing, 0, objColumn.Width - objGrid.CustomSpacing * 2 - objGrid.CustomGridSize, objGrid.CustomFooterHeight)
			'--- Background Footer Panel
			Dim pnlSepFooter As Panel: pnlSepFooter.Initialize("")
			objGrid.FooterPanel.AddView(pnlSepFooter, intPosX, 0, objColumn.Width - objGrid.CustomGridSize, objGrid.CustomFooterHeight)
			SetNinePatchDrawable(pnlSepFooter, RECORD_FOOTER): pnlSepFooter.SendToBack
			'--- Calculate Next Header Position
			intPosX = intPosX + objColumn.Width
			If intColumn < objGrid.Columns.Size - 1 Then intPosX = intPosX '+ 1dip
		End If
	Next
End Sub
#End Region

#Region Extra Grid Functions
'--- Handy sub to set the value of your data by a where, this function will refersh the data of your grid too. 
'--- Not advisable If you Do a lot of changes at once (it will refresh the grid by every change)
Sub SetGridRowCellValue(objGrid As TGrid, WhereKey As String, WhereValue As Object, NewKey As String, NewValue As Object) As Boolean
	Dim intIndex As Int: intIndex = GetListIndex(objGrid.Data, WhereKey, WhereValue)
	If intIndex > -1 Then
		Dim objMap As Map: objMap = objGrid.Data.Get(intIndex)
		objMap.Put(NewKey, NewValue)
		RefreshGridData(objGrid)
		Return True
	Else
		Return False
	End If
End Sub

Sub SetFieldValue(objGrid As TGrid, objRow As TRow, objField As TField, Value As Object) As Boolean
	'--- Update data
	Dim objMap As Map: objMap = objGrid.Data.Get(objRow.DataIndex)
	objMap.Put(objField.Column.DisplayField, Value)
	'--- Update view
	Select Case objField.Column.ColumnType
		Case GLABEL
			Dim lblView As Label: lblView = objField.FieldView: RefreshGLabel(lblView, objField.Column, Value)
		Case GIMAGE
		Case GRADIOBUTTON
		Case GCHECKBOX
		Case GBUTTON
	End Select
	RefreshFooter(objGrid)
End Sub

'--- Handy sub to get the complete map (record) using a where. 
'--- This way you can change the complete record AND refresh the data when you're ready using RefreshGridData
Sub GetGridRecord(objGrid As TGrid, WhereKey As String, WhereValue As Object) As Map
	Dim intIndex As Int: intIndex = GetListIndex(objGrid.Data, WhereKey, WhereValue)
	If intIndex > -1 Then
		Dim objMap As Map: objMap = objGrid.Data.Get(intIndex)
		Return objMap
	Else
		Return Null
	End If
End Sub

'--- Sub to get the TRow object using a where
Sub GetGridRow(objGrid As TGrid, WhereKey As String, WhereValue As Object) As TRow
	Dim intIndex As Int: intIndex = GetListIndex(objGrid.Data, WhereKey, WhereValue)
	If intIndex > -1 Then
		Dim objRow As TRow
		If objGrid.PageSize > 0 AND objGrid.Page > 0 Then '--- Take paging into account
			objRow = objGrid.Rows.Get(intIndex Mod objGrid.PageSize)
		Else
			objRow = objGrid.Rows.Get(intIndex)
		End If
		Return objRow
	Else
		Return Null
	End If
End Sub
'--- Get the TRow object by the Data Index
Sub GetRowByDataIndex(objGrid As TGrid, DataIndex As Int) As TRow
	For intRow = 0 To objGrid.Rows.Size - 1
		Dim objRow As TRow: objRow = objGrid.Rows.Get(intRow)
		If objRow.DataIndex = DataIndex Then Return objRow
	Next
	Return Null
End Sub
'--- Get the data row by the Visible Index
Sub GetDataRowByVisibleIndex(objGrid As TGrid, VisibleIndex As Int) As Map
	For intRow = 0 To objGrid.Rows.Size - 1
		Dim objRow As TRow: objRow = objGrid.Rows.Get(intRow)
		If objRow.VisibleIndex = VisibleIndex Then
			Dim objMap As Map: objMap = objGrid.Data.Get(objRow.DataIndex)
			Return objMap
		End If 
	Next
	Return Null
End Sub
'--- Get the TField object based on the (clicked) view and the containing row
Sub GetFieldByView(objGrid As TGrid, objRow As TRow, objView As View) As TField
	For intField = 0 To objRow.Fields.Size - 1
		Dim objField As TField: objField = objRow.Fields.Get(intField)
		If objField.FieldBackground = objView OR objField.FieldView = objView Then Return objField
	Next
	Return Null
End Sub

Sub SetSelectionMode(objGrid As TGrid, SelectionMode As Int)
	objGrid.SelectionMode = SelectionMode
	RefreshSelection(objGrid)
End Sub

'--- Refreshes the current selected rows/columns of the grid
Sub RefreshSelection(objGrid As TGrid)
	ClearRowsSelection(objGrid): ClearFieldsSelection(objGrid)
	Select Case objGrid.SelectionMode
		Case SELECTMODENONE
		Case SELECTMODEROWS
			For intRow = 0 To objGrid.SelectedRows.Size - 1
				Dim objRow As TRow: objRow = objGrid.SelectedRows.Get(intRow)
				SetRowSelection(objRow, True)
			Next
		Case SELECTMODEFIELDS
			For intField = 0 To objGrid.SelectedFields.Size - 1
				Dim objField As TField: objField = objGrid.SelectedFields.Get(intField)
				SetFieldSelection(objField, True)
			Next
	End Select
End Sub

'--- Row Selection
Sub AddSelectedRow(objGrid As TGrid, objRow As TRow)
	If objGrid.SelectedRows.IndexOf(objRow) = -1 Then 
		objGrid.SelectedRows.Add(objRow)
		SetRowSelection(objRow, True)
	End If
End Sub
Sub RemoveSelectedRow(objGrid As TGrid, objRow As TRow)
	Dim intIndex As Int: intIndex = objGrid.SelectedRows.IndexOf(objRow)
	If intIndex > -1 Then
		objGrid.SelectedRows.RemoveAt(intIndex)
		SetRowSelection(objRow, False)
	End If
End Sub
Sub ToggleRowSelection(objGrid As TGrid, objRow As TRow)
	Dim intIndex As Int: intIndex = objGrid.SelectedRows.IndexOf(objRow)
	If intIndex = -1 Then
		AddSelectedRow(objGrid, objRow)
	Else
		RemoveSelectedRow(objGrid, objRow)
	End If
End Sub
Sub SetRowSelection(objRow As TRow, Selected As Boolean)
	For intField = 0 To objRow.Fields.Size - 1
		Dim objField As TField: objField = objRow.Fields.Get(intField)
		SetFieldSelection(objField, Selected)
	Next
End Sub

'--- Field Selection
Sub AddSelectedField(objGrid As TGrid, objField As TField)
	If objGrid.SelectedFields.IndexOf(objField) = -1 Then 
		objGrid.SelectedFields.Add(objField)
		SetFieldSelection(objField, True)
	End If
End Sub
Sub RemoveSelectedField(objGrid As TGrid, objField As TField)
	Dim intIndex As Int: intIndex = objGrid.SelectedFields.IndexOf(objField)
	If intIndex > -1 Then
		objGrid.SelectedFields.RemoveAt(intIndex)
		SetFieldSelection(objField, False)
	End If
End Sub
Sub ToggleFieldSelection(objGrid As TGrid, objField As TField)
	Dim intIndex As Int: intIndex = objGrid.SelectedFields.IndexOf(objField)
	If intIndex = -1 Then
		AddSelectedField(objGrid, objField)
	Else
		RemoveSelectedField(objGrid, objField)
	End If
End Sub
Sub SetFieldSelection(objField As TField, Selected As Boolean)
	If objField.FieldBackground = Null Then Return
	If Selected = True Then
		SetNinePatchButton2(objField.FieldBackground, RECORD_DOWN, RECORD_DOWN, RECORD_DISABLED, RECORD_DOWN, RECORD_DISABLED)
	Else
		If objField.IsAlternate = False Then: SetNinePatchButton2(objField.FieldBackground, RECORD_ODD, RECORD_DOWN, RECORD_DISABLED, RECORD_DOWN, RECORD_DISABLED)
		Else: SetNinePatchButton2(objField.FieldBackground, RECORD_EVEN, RECORD_DOWN, RECORD_DISABLED, RECORD_DOWN, RECORD_DISABLED)
		End If
	End If
End Sub


'--- Clear Selections
Sub ClearRowsSelection(objGrid As TGrid)
	For intRow = 0 To objGrid.SelectedRows.Size - 1
		Dim objRow As TRow: objRow = objGrid.SelectedRows.Get(intRow)
		SetRowSelection(objRow, False)
	Next
End Sub
Sub ClearFieldsSelection(objGrid As TGrid)
	For intField = 0 To objGrid.SelectedFields.Size - 1
		Dim objField As TField: objField = objGrid.SelectedFields.Get(intField)
		SetFieldSelection(objField, False)
	Next
End Sub

#End Region

#End Region

'--- Simple List routines
#Region List
'--- Get the index within a given list for a specific field and value 
Sub GetListIndex(Values As List, Key As String, Value As Object) As Int
	If Values.Size = 0 Then Return -1
	For i = 0 To Values.Size - 1
		Dim objMap As Map: objMap = Values.Get(i)
		If objMap.Get(Key) = Value Then
			Return i
		End If
	Next
	Return -1
End Sub
#End Region

'--- Sorting
#Region Sorting
Sub QuickSort(Data As List, StartIndex As Long, EndIndex As Long, FieldName As String)
	If StartIndex >= EndIndex Then Return
	Dim intLow, intHigh As Int: Dim objTemp, objTestElement As Map
	intLow = StartIndex: intHigh = EndIndex
	Dim lngMiddle As Long: lngMiddle = (StartIndex + EndIndex) / 2
	objTestElement = Data.Get(lngMiddle)
	Dim objTestValue As Object: objTestValue = objTestElement.Get(FieldName)

	Do While intLow <= intHigh
		Dim tmpCurrent As Map: tmpCurrent = Data.Get(intLow): Dim tmpCurrentValue As Object: tmpCurrentValue = tmpCurrent.Get(FieldName)
		Do While IsLessThan(tmpCurrentValue, objTestValue) = True
			intLow = intLow + 1
			tmpCurrent = Data.Get(intLow): tmpCurrentValue = tmpCurrent.Get(FieldName)
		Loop

		tmpCurrent = Data.Get(intHigh): tmpCurrentValue = tmpCurrent.Get(FieldName)
		Do While IsBiggerThan(tmpCurrentValue, objTestValue) = True
			intHigh = intHigh - 1
			tmpCurrent = Data.Get(intHigh): tmpCurrentValue = tmpCurrent.Get(FieldName)
		Loop
	
		If intLow <= intHigh Then
			objTemp = Data.Get(intLow)
			Data.Set(intLow, Data.Get(intHigh))
			Data.Set(intHigh, objTemp)
			intLow = intLow + 1
			intHigh = intHigh - 1
		End If
	Loop
	
	If StartIndex < intHigh Then
		QuickSort(Data, StartIndex, intHigh, FieldName)
	End If
	
	If intLow < EndIndex Then
		QuickSort(Data, intLow, EndIndex, FieldName)
	End If
End Sub
Sub IsLessThan(Value1 As Object, Value2 As Object) As Boolean
	If Value1 Is String Then
		Dim strValue1, strValue2 As String: strValue1 = Value1: strValue2 = Value2
		Return strValue1.ToLowerCase.CompareTo(strValue2.ToLowerCase) < 0
	Else If Value1 Is Byte OR Value1 Is Int OR Value1 Is Long Then
		Dim lngValue1, lngValue2 As Long: lngValue1 = Value1: lngValue2 = Value2
		Return lngValue1 < lngValue2
	Else If Value1 Is Float OR Value1 Is Double Then
		Dim dblValue1, dblValue2 As Double: dblValue1 = Value1: dblValue2 = Value2
		Return dblValue1 < dblValue2
	Else If Value1 Is Boolean Then
		Dim strValue1, strValue2 As String: strValue1 = Value1: strValue2 = Value2
		Return strValue1.ToLowerCase.CompareTo(strValue2.ToLowerCase) < 0
	End If
End Sub
Sub IsBiggerThan(Value1 As Object, Value2 As Object) As Boolean
	If Value1 Is String Then
		Dim strValue1, strValue2 As String: strValue1 = Value1: strValue2 = Value2
		Return strValue1.ToLowerCase.CompareTo(strValue2.ToLowerCase) > 0
	Else If Value1 Is Byte OR Value1 Is Int OR Value1 Is Long Then
		Dim lngValue1, lngValue2 As Long: lngValue1 = Value1: lngValue2 = Value2
		Return lngValue1 > lngValue2
	Else If Value1 Is Float OR Value1 Is Double Then
		Dim dblValue1, dblValue2 As Double: dblValue1 = Value1: dblValue2 = Value2
		Return dblValue1 > dblValue2
	Else If Value1 Is Boolean Then
		Dim strValue1, strValue2 As String: strValue1 = Value1: strValue2 = Value2
		Return strValue1.ToLowerCase.CompareTo(strValue2.ToLowerCase) > 0
	End If
End Sub
#End Region

'--- Calculating Text Height stuff
#Region Calcualte Text Height
'--- This sub is called to get the default font size of a label. 
'--- It Is later used To calculate the exact font 'height' for multiline fields
Sub GetDefaultFontSize() As Int
	Dim lblDefault As Label: lblDefault.Initialize("")
	Return lblDefault.TextSize
End Sub
'--- Get the actual text height for a given font size
Sub GetTextHeight(objCanvas As Canvas, NewFontSize As Int) As Float
	Return objCanvas.MeasureStringHeight("Mg", Typeface.DEFAULT, NewFontSize)
End Sub

'--- Get the height of a specific View, for a specific TypeFace
'--- This method is an estimate and not rocket science
Sub GetMultiLineHeight(objCanvas As Canvas, objView As View, objTypeFace As Typeface) As Int
	Dim intWidth As Int: Dim strText As String: Dim fltFontSize As Float
	If objView Is RadioButton Then
		Dim objRadio As RadioButton: objRadio = objView: intWidth = objRadio.Width - 56dip: strText = objRadio.Text: fltFontSize = objRadio.TextSize
	Else If objView Is CheckBox Then
		Dim objCheck As CheckBox: objCheck = objView: intWidth = objCheck.Width - 56dip: strText = objCheck.Text: fltFontSize = objCheck.TextSize
	Else If objView Is EditText Then
		Dim objEdit As EditText: objEdit = objView: intWidth = objEdit.Width: strText = objEdit.Text: fltFontSize = objEdit.TextSize
	Else If objView Is Button Then
		Dim objButton As Button: objButton = objView: intWidth = objButton.Width: strText = objButton.Text: fltFontSize = objButton.TextSize
	Else
		Dim objLabel As Label: objLabel = objView: intWidth = objLabel.Width: strText = objLabel.Text: fltFontSize = objLabel.TextSize
	End If
	Dim fltTotalWidth As Float: fltTotalWidth = objCanvas.MeasureStringWidth(strText, Typeface.DEFAULT, fltFontSize)
	Dim fltHeight As Float: fltHeight = GetTextHeight(objCanvas, fltFontSize) + PADDING
	If intWidth = 0 Then intWidth = 1
	Dim fltNumLines As Float: fltNumLines = fltTotalWidth / intWidth
	For i = 0 To strText.Length - 1
		If strText.CharAt(i) = Chr(10) Then fltNumLines = fltNumLines + 1
	Next
	Dim intNumLines As Int: intNumLines = fltNumLines: intNumLines = intNumLines + 1
	Return intNumLines * fltHeight
End Sub
#End Region

'--- These routines are used to clear all views from a given parent
#Region Clear
Sub ClearView(objView As View, ResetHeight As Boolean)
	If objView Is ScrollView Then
		Dim objScrollView As ScrollView: objScrollView = objView
		ClearScrollView(objScrollView)
	Else If objView Is HorizontalScrollView Then
		Dim objHorizontalScrollView As HorizontalScrollView: objHorizontalScrollView = objView
		ClearHorizontalScrollView(objHorizontalScrollView)
	Else If objView Is Panel Then
		Dim objPanel As Panel: objPanel = objView
		ClearPanel(objPanel, False, ResetHeight)
	End If
End Sub

Sub ClearPanel(objPanel As Panel, RemovePanel As Boolean, ResetHeight As Boolean)
	If objPanel.NumberOfViews > 0 Then
		Do While objPanel.NumberOfViews > 0
			Dim objView As View: objView = objPanel.GetView(0)
			ClearView(objView, ResetHeight)
			objPanel.RemoveViewAt(0)
		Loop
		If ResetHeight = True Then objPanel.Height = 0
	End If
	If RemovePanel = True Then objPanel.RemoveView
End Sub
Sub ClearScrollView(vscList As ScrollView)
	If vscList.Panel.NumberOfViews > 0 Then
		Do While vscList.Panel.NumberOfViews > 0
			Dim objView As View: objView = vscList.Panel.GetView(0)
			ClearView(objView, True)
			vscList.Panel.RemoveViewAt(0)
		Loop
		vscList.Panel.Height = 0
	End If
End Sub
Sub ClearHorizontalScrollView(hscList As HorizontalScrollView)
	If hscList.Panel.NumberOfViews > 0 Then
		Do While hscList.Panel.NumberOfViews > 0
			Dim objView As View: objView = hscList.Panel.GetView(0)
			ClearView(objView, True)
			hscList.Panel.RemoveViewAt(0)
		Loop
		hscList.Panel.Width = 0
	End If
End Sub
#End Region

'--- See Erel's thread about nine patch images for more info
#Region Nine Patch Image
Sub SetNinePatchDrawable(Control As View, ImageName As String)
    Dim r As Reflector
    Dim package As String
    Dim id As Int
    package = r.GetStaticField("anywheresoftware.b4a.BA", "packageName")
    id = r.GetStaticField(package & ".R$drawable", ImageName)
    r.Target = r.GetContext
    r.Target = r.RunMethod("getResources")
    Control.Background = r.RunMethod2("getDrawable", id, "java.lang.int")
End Sub
Sub SetNinePatchButton(Btn As Button, DefaultImage As String, PressedImage As String)
    Dim r As Reflector
    Dim package As String
    Dim idDefault, idPressed As Int
    package = r.GetStaticField("anywheresoftware.b4a.BA", "packageName")
    idDefault = r.GetStaticField(package & ".R$drawable", DefaultImage)
    idPressed = r.GetStaticField(package & ".R$drawable", PressedImage)
    r.Target = r.GetContext
    r.Target = r.RunMethod("getResources")
    Dim sd As StateListDrawable
    sd.Initialize
    sd.AddState(sd.State_Pressed, r.RunMethod2("getDrawable", idPressed, "java.lang.int"))
    sd.AddCatchAllState( r.RunMethod2("getDrawable", idDefault, "java.lang.int"))
    Btn.Background = sd
End Sub
Sub SetNinePatchButton2(Btn As Button, DefaultImage As String, PressedImage As String, _
    DisabledImage As String, FocusedImage As String, FocusedDisabledImage As String)

    Dim r As Reflector
    Dim package As String
    package = r.GetStaticField("anywheresoftware.b4a.BA", "packageName")
    
    r.Target = r.GetContext
    r.Target = r.RunMethod("getResources")
    Dim sd As StateListDrawable
    sd.Initialize
    
    Dim idDefault, idPressed, idFocused, idDisabled, idFocusedDisabled As Int
    
    If FocusedDisabledImage.Length > 0 Then
        idFocusedDisabled = r.GetStaticField(package & ".R$drawable", FocusedDisabledImage)
        sd.AddState2(Array As Int (sd.State_Disabled, 0x0101009c), r.RunMethod2("getDrawable", idFocusedDisabled, "java.lang.int"))
    End If
    If DisabledImage.Length > 0 Then
        idDisabled = r.GetStaticField(package & ".R$drawable", DisabledImage)
        sd.AddState(sd.State_Disabled, r.RunMethod2("getDrawable", idDisabled, "java.lang.int"))
    End If
    If PressedImage.Length > 0 Then
        idPressed = r.GetStaticField(package & ".R$drawable", PressedImage)
        sd.AddState(sd.State_Pressed, r.RunMethod2("getDrawable", idPressed, "java.lang.int"))
    End If
    If FocusedImage.Length > 0 Then
        idFocused = r.GetStaticField(package & ".R$drawable", FocusedImage)
        sd.AddState(0x0101009c, r.RunMethod2("getDrawable", idFocused, "java.lang.int"))
    End If
    If DefaultImage.Length > 0 Then
        idDefault = r.GetStaticField(package & ".R$drawable", DefaultImage)
        sd.AddCatchAllState( r.RunMethod2("getDrawable", idDefault, "java.lang.int"))
    End If
    
    Btn.Background = sd
End Sub
#End Region

'--- Drawing routines
#Region Drawing
'--- This can be called to set AntiAlias on for a given canvas
'--- The result is smoother UI
Sub SetAntiAlias (objCanvas As Canvas)
    Dim r As Reflector
    Dim NativeCanvas As Object
    r.Target = objCanvas
    NativeCanvas = r.GetField("canvas")
    Dim PaintFlagsDrawFilter As Object
    PaintFlagsDrawFilter = r.CreateObject2("android.graphics.PaintFlagsDrawFilter", Array As Object(0, 1), Array As String("java.lang.int", "java.lang.int"))
    r.Target = NativeCanvas
    r.RunMethod4("setDrawFilter", Array As Object(PaintFlagsDrawFilter), Array As String("android.graphics.DrawFilter"))
End Sub
#End Region
